#!/usr/bin/env python
import json
#import adal
#import azure.common.credentials as credentials
#import azure.batch._batch_service_client as batch
#import azure.batch.batch_auth as batchauth
#import azure.batch.models as batchmodels
#from azure.servicebus import SubscriptionClient, ServiceBusClient, TopicClient, QueueClient, Message
from azure.servicebus import ServiceBusReceiver, ServiceBusReceiveMode, ServiceBusClient
#from typing_extensions import Required
from dhi.platform.args import ClientArgs
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("-c", "--connectionstring", help="Service bus connection string", required=True)
    parser.add_argument("-x", "--executionid", default=ClientArgs.GetDefault("DHIEXECUTIONID"), help="Execution run id (topic name)")
    parser.add_argument("-s", "--subscription", default="engine", help="Subscription name")
    parser.add_argument("-t", "--timeout", default=10, help="Timeout [seconds]", type=int)

def main():
    args = ClientArgs.ParseBasic(description="Show engine messages", init=initParser)

    #topicClient = TopicClient.from_connection_string(connectionString, topicName)
    #client = ServiceBusClient.from_connection_string(connectionString)
    #subscription = client.get_subscription(topicName, subscriptionName)
    #subscription = SubscriptionClient.from_connection_string(args.connectionstring, args.subscription, topic=args.executionid)
    #for msg in subscription.get_receiver(idle_timeout=args.timeout):
    #    print(msg)
    #    msg.complete
    #receiver = ServiceBusReceiver._from_connection_string(conn_str=args.connectionstring, topic_name=args.executionid, #subscription_name=args.subscription, receive_mode=ServiceBusReceiveMode.PEEK_LOCK, max_wait_time=args.timeout)
    #if receiver:
    #    for msg in receiver:
    #        Format.DumpPlain(vars(msg))

    client = ServiceBusClient.from_connection_string(conn_str=args.connectionstring)
    with client:
        receiver = client.get_subscription_receiver(topic_name=args.executionid, subscription_name=args.subscription, receive_mode=ServiceBusReceiveMode.PEEK_LOCK, max_wait_time=args.timeout) #, prefetch_count=20)
        with receiver:
            for msg in receiver:
                print(f"### {msg.message_id}")
                #print(f"  {type(msg.message)}")
                #print(f"  {msg.message}")
                #obj = json.loads(f"{msg}")
                obj = json.loads(str(msg))
                Format.DumpPlain(obj)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.metadatahelper import MetadataClientV3Contracts
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--dformat", default="file", help="Format")
    parser.add_argument("--srid", type=int, help="SRID")
    parser.add_argument("--arguments", help="Arguments")
    parser.add_argument("datasetid", help="Dataset id")

def main():
    args = ClientArgs.ParsePlatform(description="Download file", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    input = MetadataClientV3Contracts.PrepareDownloadDatasetInput(
        ClientArgs.LoadJson(args.inputfile),
        args.dformat,
        args.srid,
        ClientArgs.LoadJsonStr(args.arguments))
    response = clientv3.DownloadDatasetV3(input, args.datasetid)

    Format.FormatResponse(response, lambda r: r.Body, args.format)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig


def __InitParser(parser):
    parser.add_argument("-i", "--login", help="Interactive login", action="store_true")
    parser.add_argument("--refresh", help="Refresh access token from previous login", action="store_true")
    parser.add_argument("--preferapikey", help="Prefere api key", action="store_true")
    parser.add_argument("--prefertoken", help="Prefere access token", action="store_true")
    parser.add_argument("--show", help="Show login information", action="store_true")
    parser.add_argument("--clearlogin", help="Clear login information", action="store_true")
    parser.add_argument("--clearapikey", help="Clear apikey information", action="store_true")

def main():
    args = ClientArgs.ParseApiKey(description="Interactive login", init=__InitParser)
    ClientConfig.UpdateEnvironmentFromConfiguration(args, args.config)

    if args.show:
        print(f"Configuration from {ClientConfig.GetConfigurationFilePath(args.config)}")
        envObj = ClientConfig.GetEnvironment(args.environment, args.config)
        tokeninfo = ClientConfig.GetAccessTokenInfo(envObj)
        fmt = "{!s:12}\t{}"
        print(fmt.format("environment", args.environment))
        print(fmt.format("username", ClientConfig.GetUsername(tokeninfo)))
        print(fmt.format("expires", ClientConfig.GetExpiresOn(tokeninfo)))
        print(fmt.format("isexpired", ClientConfig.IsExpired(tokeninfo)))
        print(fmt.format("apikey", ClientConfig.GetApiKey(envObj)))
        print(fmt.format("preferapikey", ClientConfig.GetPreferApiKey(envObj)))
    elif args.clearlogin or args.clearapikey:
        ClientConfig.ClearInfo(args.environment, clearlogin=args.clearlogin, clearapikey=args.clearapikey, configname=args.config)
    else:
        if args.login:
            from dhi.platform.authentication import ClientAuthentication
            (tokeninfo, _) = ClientAuthentication.AcquireTokenInteractively(args.environment)
            if tokeninfo:
                ClientConfig.SaveUserTokenInfo(args.environment, tokeninfo, args.config)
                username = tokeninfo.get("username")
                expireson = ClientConfig.GetExpiresOn(tokeninfo)
                fmt = "{!s:8}\t{}"
                print(fmt.format("environment", args.environment))
                print(fmt.format("username", username))
                print(fmt.format("expires", expireson))
        elif args.refresh:
            from dhi.platform.authentication import ClientAuthentication
            envObj = ClientConfig.GetEnvironment(args.environment, args.config)
            tokeninfo = ClientConfig.GetAccessTokenInfo(envObj)
            if not tokeninfo:
                print("ERROR: No access token to renew!")
            else:
                (tokeninfo, _) = ClientAuthentication.RefreshToken(args.environment, tokeninfo)
                if tokeninfo:
                    ClientConfig.SaveUserTokenInfo(args.environment, tokeninfo, args.config)
                    username = tokeninfo.get("username")
                    expireson = ClientConfig.GetExpiresOn(tokeninfo)
                    fmt = "{!s:8}\t{}"
                    print(fmt.format("environment", args.environment))
                    print(fmt.format("username", username))
                    print(fmt.format("expires", expireson))
                    print(fmt.format("isexpired", ClientConfig.IsExpired(tokeninfo)))
        elif args.prefertoken:
            ClientConfig.SetPreferApiKey(args.environment, False, args.config)
            envObj = ClientConfig.GetEnvironment(args.environment, args.config)
            tokeninfo = ClientConfig.GetAccessTokenInfo(envObj)
            if tokeninfo:
                username = tokeninfo.get("username")
                expireson = ClientConfig.GetExpiresOn(tokeninfo)
                fmt = "{!s:8}\t{}"
                print(fmt.format("environment", args.environment))
                print(fmt.format("username", username))
                print(fmt.format("expires", expireson))
                print(fmt.format("isexpired", ClientConfig.IsExpired(tokeninfo)))
        elif args.apikey:
            ClientConfig.SaveApiKey(args.environment, args.apikey, args.config)
            print(f"apikey\t{args.apikey}")
        elif args.preferapikey:
            ClientConfig.SetPreferApiKey(args.environment, True, args.config)
            envObj = ClientConfig.GetEnvironment(args.environment)
            print(f"apikey\t{ClientConfig.GetApiKey(envObj)}")

if __name__ == '__main__':
    main()



# Generated using openapi2py.py
# openapi2py.py "--dhiservice" "gis" "--classname" "GisGenClientV" "-n" "2" "-r" "projectid" "-i" "datasetid=id" "--out" "C:\work\devops\mike-platform-sdk-py\src\dhi\platform\generated\gisgen.py" "https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/gis/v2"
# 2022-01-13 19:02:32.131166Z
from typing import Any, Dict, List, NewType, Type, TypeVar, Union
from enum import Enum
import attr
from ..base.client import DataContract, PlatformClient, Response


# https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/gis/v2
# DHI Water Data GIS API - Version 2
# API for managing features, feature attributes and feature collections
# 2

StringCollectionResponseV2Type = TypeVar("StringCollectionResponseV2Type", bound="StringCollectionResponseV2")

@attr.s(auto_attribs=True)
class StringCollectionResponseV2(DataContract):
    data: List[str] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: StringCollectionResponseV2Type, src_dict: Dict[str, Any]) -> StringCollectionResponseV2Type:
        obj = StringCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

SelectClauseV2Type = TypeVar("SelectClauseV2Type", bound="SelectClauseV2")

@attr.s(auto_attribs=True)
class SelectClauseV2(DataContract):
    outputSRID: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: SelectClauseV2Type, src_dict: Dict[str, Any]) -> SelectClauseV2Type:
        obj = SelectClauseV2()
        obj.load_dict(src_dict)
        return obj

SelectAllAttributesV2Type = TypeVar("SelectAllAttributesV2Type", bound="SelectAllAttributesV2")

@attr.s(auto_attribs=True)
class SelectAllAttributesV2(SelectClauseV2):
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = SelectClauseV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: SelectAllAttributesV2Type, src_dict: Dict[str, Any]) -> SelectAllAttributesV2Type:
        obj = SelectAllAttributesV2()
        obj.load_dict(src_dict)
        return obj

ProjectOutputV2Type = TypeVar("ProjectOutputV2Type", bound="ProjectOutputV2")

@attr.s(auto_attribs=True)
class ProjectOutputV2(DataContract):
    inputSrid: int = None
    outputSrid: int = None
    geometries: List[str] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ProjectOutputV2Type, src_dict: Dict[str, Any]) -> ProjectOutputV2Type:
        obj = ProjectOutputV2()
        obj.load_dict(src_dict)
        return obj

DeleteAttributeInputV2Type = TypeVar("DeleteAttributeInputV2Type", bound="DeleteAttributeInputV2")

@attr.s(auto_attribs=True)
class DeleteAttributeInputV2(DataContract):
    name: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: DeleteAttributeInputV2Type, src_dict: Dict[str, Any]) -> DeleteAttributeInputV2Type:
        obj = DeleteAttributeInputV2()
        obj.load_dict(src_dict)
        return obj

QueryConditionV2Type = TypeVar("QueryConditionV2Type", bound="QueryConditionV2")

@attr.s(auto_attribs=True)
class QueryConditionV2(DataContract):
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: QueryConditionV2Type, src_dict: Dict[str, Any]) -> QueryConditionV2Type:
        obj = QueryConditionV2()
        obj.load_dict(src_dict)
        return obj

FeatureQueryV2Type = TypeVar("FeatureQueryV2Type", bound="FeatureQueryV2")

@attr.s(auto_attribs=True)
class FeatureQueryV2(DataContract):
    selectClause: SelectClauseV2 = None
    conditions: List[QueryConditionV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: FeatureQueryV2Type, src_dict: Dict[str, Any]) -> FeatureQueryV2Type:
        obj = FeatureQueryV2()
        obj.load_dict(src_dict)
        return obj

ProjectInputV2Type = TypeVar("ProjectInputV2Type", bound="ProjectInputV2")

@attr.s(auto_attribs=True)
class ProjectInputV2(DataContract):
    inputSrid: int = None
    outputSrid: int = None
    geometries: List[str] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ProjectInputV2Type, src_dict: Dict[str, Any]) -> ProjectInputV2Type:
        obj = ProjectInputV2()
        obj.load_dict(src_dict)
        return obj

class SpatialOperatorV2(str, Enum):
    INTERSECTS = "Intersects"
    WITHIN = "Within"
    def __str__(self) -> str:
        return str(self.value)

SpatialQueryConditionV2Type = TypeVar("SpatialQueryConditionV2Type", bound="SpatialQueryConditionV2")

@attr.s(auto_attribs=True)
class SpatialQueryConditionV2(QueryConditionV2):
    geometry: str = None
    operator: SpatialOperatorV2 = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: SpatialQueryConditionV2Type, src_dict: Dict[str, Any]) -> SpatialQueryConditionV2Type:
        obj = SpatialQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

ProblemDetailsV2Type = TypeVar("ProblemDetailsV2Type", bound="ProblemDetailsV2")

@attr.s(auto_attribs=True)
class ProblemDetailsV2(DataContract):
    type: str = None
    title: str = None
    status: int = None
    detail: str = None
    instance: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ProblemDetailsV2Type, src_dict: Dict[str, Any]) -> ProblemDetailsV2Type:
        obj = ProblemDetailsV2()
        obj.load_dict(src_dict)
        return obj

ValidationProblemDetailsV2Type = TypeVar("ValidationProblemDetailsV2Type", bound="ValidationProblemDetailsV2")

@attr.s(auto_attribs=True)
class ValidationProblemDetailsV2(ProblemDetailsV2):
    errors: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = ProblemDetailsV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ValidationProblemDetailsV2Type, src_dict: Dict[str, Any]) -> ValidationProblemDetailsV2Type:
        obj = ValidationProblemDetailsV2()
        obj.load_dict(src_dict)
        return obj

class ComparisonOperatorV2(str, Enum):
    EQUAL = "Equal"
    NOTEQUAL = "NotEqual"
    GREATERTHAN = "GreaterThan"
    LESSTHAN = "LessThan"
    GREATERTHANOREQUAL = "GreaterThanOrEqual"
    LESSTHANOREQUAL = "LessThanOrEqual"
    def __str__(self) -> str:
        return str(self.value)

class AttributeOperatorV2(str, Enum):
    EQUAL = "Equal"
    NOTEQUAL = "NotEqual"
    GREATERTHAN = "GreaterThan"
    LESSTHAN = "LessThan"
    GREATERTHANOREQUAL = "GreaterThanOrEqual"
    LESSTHANOREQUAL = "LessThanOrEqual"
    CONTAINS = "Contains"
    STARTSWITH = "StartsWith"
    def __str__(self) -> str:
        return str(self.value)

AttributeQueryConditionV2Type = TypeVar("AttributeQueryConditionV2Type", bound="AttributeQueryConditionV2")

@attr.s(auto_attribs=True)
class AttributeQueryConditionV2(QueryConditionV2):
    name: str = None
    operator: AttributeOperatorV2 = None
    value: None = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: AttributeQueryConditionV2Type, src_dict: Dict[str, Any]) -> AttributeQueryConditionV2Type:
        obj = AttributeQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

IdsQueryConditionV2Type = TypeVar("IdsQueryConditionV2Type", bound="IdsQueryConditionV2")

@attr.s(auto_attribs=True)
class IdsQueryConditionV2(QueryConditionV2):
    ids: List[int] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: IdsQueryConditionV2Type, src_dict: Dict[str, Any]) -> IdsQueryConditionV2Type:
        obj = IdsQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

FullTextQueryConditionV2Type = TypeVar("FullTextQueryConditionV2Type", bound="FullTextQueryConditionV2")

@attr.s(auto_attribs=True)
class FullTextQueryConditionV2(QueryConditionV2):
    searchString: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: FullTextQueryConditionV2Type, src_dict: Dict[str, Any]) -> FullTextQueryConditionV2Type:
        obj = FullTextQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

CreatedAtQueryConditionV2Type = TypeVar("CreatedAtQueryConditionV2Type", bound="CreatedAtQueryConditionV2")

@attr.s(auto_attribs=True)
class CreatedAtQueryConditionV2(QueryConditionV2):
    createdAt: str = None
    operator: ComparisonOperatorV2 = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: CreatedAtQueryConditionV2Type, src_dict: Dict[str, Any]) -> CreatedAtQueryConditionV2Type:
        obj = CreatedAtQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

IFeatureV2Type = TypeVar("IFeatureV2Type", bound="IFeatureV2")

@attr.s(auto_attribs=True)
class IFeatureV2(DataContract):
    id: int = None
    geometry: str = None
    attributes: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: IFeatureV2Type, src_dict: Dict[str, Any]) -> IFeatureV2Type:
        obj = IFeatureV2()
        obj.load_dict(src_dict)
        return obj

CoordinateSystemOutputV2Type = TypeVar("CoordinateSystemOutputV2Type", bound="CoordinateSystemOutputV2")

@attr.s(auto_attribs=True)
class CoordinateSystemOutputV2(DataContract):
    id: int = None
    name: str = None
    authority: str = None
    wkt: str = None
    proj4String: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: CoordinateSystemOutputV2Type, src_dict: Dict[str, Any]) -> CoordinateSystemOutputV2Type:
        obj = CoordinateSystemOutputV2()
        obj.load_dict(src_dict)
        return obj

CoordinateSystemOutputCollectionResponseV2Type = TypeVar("CoordinateSystemOutputCollectionResponseV2Type", bound="CoordinateSystemOutputCollectionResponseV2")

@attr.s(auto_attribs=True)
class CoordinateSystemOutputCollectionResponseV2(DataContract):
    data: List[CoordinateSystemOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: CoordinateSystemOutputCollectionResponseV2Type, src_dict: Dict[str, Any]) -> CoordinateSystemOutputCollectionResponseV2Type:
        obj = CoordinateSystemOutputCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

SelectCustomAttributesV2Type = TypeVar("SelectCustomAttributesV2Type", bound="SelectCustomAttributesV2")

@attr.s(auto_attribs=True)
class SelectCustomAttributesV2(SelectClauseV2):
    attributes: List[str] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = SelectClauseV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: SelectCustomAttributesV2Type, src_dict: Dict[str, Any]) -> SelectCustomAttributesV2Type:
        obj = SelectCustomAttributesV2()
        obj.load_dict(src_dict)
        return obj

class DatasetTypeV2(str, Enum):
    FILE = "file"
    MULTIDIMENSIONAL = "multidimensional"
    TIMESERIES = "timeseries"
    GISVECTORDATA = "gisvectordata"
    TILES = "tiles"
    def __str__(self) -> str:
        return str(self.value)

DatasetTypeQueryConditionV2Type = TypeVar("DatasetTypeQueryConditionV2Type", bound="DatasetTypeQueryConditionV2")

@attr.s(auto_attribs=True)
class DatasetTypeQueryConditionV2(QueryConditionV2):
    datasetType: DatasetTypeV2 = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: DatasetTypeQueryConditionV2Type, src_dict: Dict[str, Any]) -> DatasetTypeQueryConditionV2Type:
        obj = DatasetTypeQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

class AttributeDataTypeV2(str, Enum):
    TEXT = "Text"
    DATE = "Date"
    INT32 = "Int32"
    INT64 = "Int64"
    SINGLE = "Single"
    DOUBLE = "Double"
    INT16 = "Int16"
    def __str__(self) -> str:
        return str(self.value)

IFeatureAttributeV2Type = TypeVar("IFeatureAttributeV2Type", bound="IFeatureAttributeV2")

@attr.s(auto_attribs=True)
class IFeatureAttributeV2(DataContract):
    dataType: AttributeDataTypeV2 = None
    defaultValue: None = None
    label: str = None
    length: int = None
    name: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: IFeatureAttributeV2Type, src_dict: Dict[str, Any]) -> IFeatureAttributeV2Type:
        obj = IFeatureAttributeV2()
        obj.load_dict(src_dict)
        return obj

IFeatureClassV2Type = TypeVar("IFeatureClassV2Type", bound="IFeatureClassV2")

@attr.s(auto_attribs=True)
class IFeatureClassV2(DataContract):
    attributes: List[IFeatureAttributeV2] = None
    features: List[IFeatureV2] = None
    id: str = None
    srid: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: IFeatureClassV2Type, src_dict: Dict[str, Any]) -> IFeatureClassV2Type:
        obj = IFeatureClassV2()
        obj.load_dict(src_dict)
        return obj

CatalogIdQueryConditionV2Type = TypeVar("CatalogIdQueryConditionV2Type", bound="CatalogIdQueryConditionV2")

@attr.s(auto_attribs=True)
class CatalogIdQueryConditionV2(QueryConditionV2):
    catalogId: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = QueryConditionV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: CatalogIdQueryConditionV2Type, src_dict: Dict[str, Any]) -> CatalogIdQueryConditionV2Type:
        obj = CatalogIdQueryConditionV2()
        obj.load_dict(src_dict)
        return obj

class GisGenClientV2(PlatformClient):
    def __init__(self, inspectFnc=PlatformClient.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, includeheaders=PlatformClient.GetServiceHeaders("gis"), **kwargs)

    def GetAuthorities(self, projectid) -> Response:
        """Gets list of unique authorities of all coordinate systems

        CoordinateSystem
        GET /api/crs/coordinate-system/authority
        """
        return self.GetRequest("/api/crs/coordinate-system/authority", None, api_version="2", projectid=projectid)

    def GetAllCoordinateSystems(self, projectid, nametext=None, fulltext=None, authority=None) -> Response:
        """Gets list of coordinate systems with optional filters

        CoordinateSystem
        GET /api/crs/coordinate-system/list
        """
        queryparams = self.GetQueryParams(nametext=nametext, fulltext=fulltext, authority=authority)
        return self.GetRequest("/api/crs/coordinate-system/list", queryparams, api_version="2", projectid=projectid)

    def GetCoordinateSystem(self, projectid, id) -> Response:
        """Gets coordinate system by spatial reference id

        CoordinateSystem
        GET /api/crs/coordinate-system/{id}
        """
        return self.GetRequest(f"/api/crs/coordinate-system/{id}", None, api_version="2", projectid=projectid, datasetid=id)

    def GetProjectedGeometries(self, projectid, body) -> Response:
        """Returns geometries projected to a selected coordinate system

        CoordinateSystem
        POST /api/crs/project
        """
        return self.PostRequest("/api/crs/project", body, None, api_version="2", projectid=projectid)

    def GetFeatureClassAsFeatureCollection(self, projectid, id, outputsrid=None) -> Response:
        """Gets a feature class as feature collection

        FeatureClass
        GET /api/gis/dataset/{id}
        """
        queryparams = self.GetQueryParams(outputSRID=outputsrid)
        return self.GetRequest(f"/api/gis/dataset/{id}", queryparams, api_version="2", projectid=projectid, datasetid=id)

    def AddAttribute(self, projectid, body, id) -> Response:
        """Add attribute to feature class

        FeatureClass
        PATCH /api/gis/dataset/{id}/add-attribute
        """
        return self.PatchRequest(f"/api/gis/dataset/{id}/add-attribute", body, None, api_version="2", projectid=projectid, datasetid=id)

    def AddFeatures(self, projectid, body, id) -> Response:
        """Adds features

        FeatureClass
        PATCH /api/gis/dataset/{id}/add-features
        """
        return self.PatchRequest(f"/api/gis/dataset/{id}/add-features", body, None, api_version="2", projectid=projectid, datasetid=id)

    def DeleteAttribute(self, projectid, body, id) -> Response:
        """Delete attribute from feature class

        FeatureClass
        PATCH /api/gis/dataset/{id}/delete-attribute
        """
        return self.PatchRequest(f"/api/gis/dataset/{id}/delete-attribute", body, None, api_version="2", projectid=projectid, datasetid=id)

    def DeleteFeatures(self, projectid, body, id) -> Response:
        """Deletes a features

        FeatureClass
        PATCH /api/gis/dataset/{id}/delete-features
        """
        return self.PatchRequest(f"/api/gis/dataset/{id}/delete-features", body, None, api_version="2", projectid=projectid, datasetid=id)

    def GetFeatureClassExtent(self, projectid, id, outputsrid=None) -> Response:
        """Gets feature class extent

        FeatureClass
        GET /api/gis/dataset/{id}/extent
        """
        queryparams = self.GetQueryParams(outputSRID=outputsrid)
        return self.GetRequest(f"/api/gis/dataset/{id}/extent", queryparams, api_version="2", projectid=projectid, datasetid=id)

    def AddFeatureV2(self, projectid, body, id) -> Response:
        """Creates a new feature

        Feature
        POST /api/gis/dataset/{id}/feature
        """
        return self.PostRequest(f"/api/gis/dataset/{id}/feature", body, None, api_version="2", projectid=projectid, datasetid=id)

    def UpdateFeatureV2(self, projectid, body, id) -> Response:
        """Updates a feature

        Feature
        PUT /api/gis/dataset/{id}/feature
        """
        return self.PutRequest(f"/api/gis/dataset/{id}/feature", body, None, api_version="2", projectid=projectid, datasetid=id)

    def DeleteFeatureV2(self, projectid, id, featureid) -> Response:
        """Deletes a feature

        Feature
        DELETE /api/gis/dataset/{id}/feature/{featureId}
        """
        return self.DeleteRequest(f"/api/gis/dataset/{id}/feature/{featureid}", None, api_version="2", projectid=projectid, datasetid=id)

    def UpdateFeatureAttributesV2(self, projectid, body, id, featureid) -> Response:
        """Updates a feature attributes

        Feature
        PATCH /api/gis/dataset/{id}/feature/{featureId}
        """
        return self.PatchRequest(f"/api/gis/dataset/{id}/feature/{featureid}", body, None, api_version="2", projectid=projectid, datasetid=id)

    def GetFeatureV2(self, projectid, id, featureid) -> Response:
        """Gets a feature

        Feature
        GET /api/gis/dataset/{id}/feature/{featureId}
        """
        return self.GetRequest(f"/api/gis/dataset/{id}/feature/{featureid}", None, api_version="2", projectid=projectid, datasetid=id)

    def QueryFeatureClass(self, projectid, body, id) -> Response:
        """Gets results of a query on a feature class as feature collection

        FeatureClass
        POST /api/gis/dataset/{id}/query
        """
        return self.PostRequest(f"/api/gis/dataset/{id}/query", body, None, api_version="2", projectid=projectid, datasetid=id)

    def UpdateFeatures(self, projectid, body, id) -> Response:
        """Updates a features

        FeatureClass
        PATCH /api/gis/dataset/{id}/update-features
        """
        return self.PatchRequest(f"/api/gis/dataset/{id}/update-features", body, None, api_version="2", projectid=projectid, datasetid=id)



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("--offset", default=0, help="Output offset")
    parser.add_argument("--limit", default=200, help="Limit output size")
    parser.add_argument("--datasettype", help="Dataset type")
    parser.add_argument("--includesastokens", help="Turn on to get SAS tokens", action="store_true")

def main():
    args = ClientArgs.ParseForProject(description="List datasets hierarchy", init=initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    response = clientv3.GetRecursiveDatasetListV3(args.projectid, args.offset, args.limit, args.datasettype, args.includesastokens)

    tablefmt = "{!s:32}\t{!s:10}\t{!s:26}\t{}"
    tablefields = ["id", "datasetType", "name", "relativePath"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



import unittest
import datetime
from unittest.case import _AssertRaisesContext
import uuid

from dhi.platform import metadata, transfer
from .testcredentials import TEST_IDENTITY
import os
import hashlib
import requests

class TestTransferTestIntegration(unittest.TestCase):

    _verbosity = 0
    _project_id = None
    
    def setUp(self) -> None:
        self._metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._transferclient = transfer.TransferClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._test_data_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
        
        if not self._project_id:
            stamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
            name = 'Python test ' + stamp
            projectInput = metadata.CreateProjectInput(name, 'Project created by Python SDK test')
            project = self._metadataclient.create_project(projectInput)
            self._project_id = project.id

            self._temp_dir = os.path.join(self._test_data_dir, 'temp' + stamp)
            os.mkdir(self._temp_dir)

    def tearDown(self) -> None:
        super().tearDown()
        if self._project_id:
            self._metadataclient.delete_project(self._project_id, permanently=True)
        
        if os.path.exists(self._temp_dir):
            import shutil
            shutil.rmtree(self._temp_dir)

    def test_get_readers_is_ok(self):
        readers = self._transferclient.get_readers()
        self.assertTrue(readers)

    def test_get_writers_is_ok(self):
        writers = self._transferclient.get_writers()
        self.assertTrue(writers)

    def test_fluent_upload_is_ok(self):
        local_file = os.path.join(self._test_data_dir, 'EqD2.dfs2')
        transfer_process = self._transferclient.create_file_import(self._project_id, local_file) \
            .with_dataset_name("PyTest") \
            .with_dataset_description("Created from python test") \
            .execute_and_wait()
        
        self.assertIsNotNone(transfer_process)
        self.assertIsNotNone(transfer_process.dataset_id)

    def test_fluent_dataset_upadte_from_dataset_is_ok(self):
        local_file = os.path.join(self._test_data_dir, 'EqD2.dfs2')

        upload1 = self._transferclient \
            .create_file_import(self._project_id, local_file) \
            .with_dfs2_reader() \
            .with_multidimensional_writer() \
            .execute_and_wait()

        upload2 = self._transferclient \
            .create_file_import(self._project_id, local_file) \
            .with_dfs2_reader() \
            .with_multidimensional_writer() \
            .execute_and_wait()

        result = self._transferclient \
            .create_dataset_update_from_dataset(upload1.dataset_id, upload2.dataset_id) \
            .execute_and_wait()
        
        self.assertTrue(isinstance(result, transfer.EmptyTransferOutput))

    def test_upload_to_multidimensional_with_allowed_items_is_ok(self):
        local_file = os.path.join(self._test_data_dir, 'EqD2.dfs2')
        reader = transfer.Dfs2Reader()
        reader.with_allowed_items("Item2")

        result = self._transferclient \
            .create_file_import(self._project_id, local_file) \
            .with_reader(reader) \
            .with_multidimensional_writer() \
            .with_coordinate_system_transformation(3857) \
            .execute_and_wait()
        
        self.assertIsNotNone(result)
        self.assertIsNotNone(result.dataset_id)

    def test_upload_staged_files_is_ok(self):
        file_name = 'EqD2.dfs2'
        local_file = os.path.join(self._test_data_dir, file_name)
        upload_folder = "Upload" + datetime.datetime.now().strftime("%Y%m%d%H%M%S")

        staging_url = self._transferclient.stage_file(local_file)
        
        input = transfer.StagedFilesUploadInput(
            files=[ transfer.StagedFileUploadInput(staging_url, file_name=file_name) ],
            destination_path=upload_folder,
            create_destination_path_if_not_exists=True
        )

        result = self._transferclient.upload_staged_files(self._project_id, input)

        self.assertEqual(len(result.datasets), 1)
        self.assertFalse(result.failures)

    def _get_remote_file_url(self, file_name):
        return f"https://coreplatformdevstor.blob.core.windows.net/int-test-input/{file_name}"

    def _download_remote_file(self, url, destination_file):
        response = requests.get(url, stream=True)
        with open(destination_file, "wb") as file:
            for chunk in response.iter_content(chunk_size=1024):
                if chunk:
                    file.write(chunk)

    def stream_large_file_in_is_ok(self):
        file_name = "CiudadDelPlata_Max.dfsu"
        sotrage_url = self._get_remote_file_url(file_name)
        local_file = os.path.join(self._temp_dir, file_name)
        processed_file = os.path.join(self._temp_dir, file_name + '.downloaded')

        self._download_remote_file(sotrage_url, local_file)

        dataset_id = None
        with open(local_file, 'rb') as stream:
            transfer_process_output = self._transferclient.create_stream_import(self._project_id, stream, file_name, file_name) \
                .with_reader_and_parameters("DfsuReader") \
                .with_writer_and_parameters("FileWriter") \
                .execute_and_wait()

            dataset_id = transfer_process_output.dataset_id

        self.assertIsNotNone(dataset_id)

        self._transferclient.download_file(self._project_id, dataset_id, processed_file)

        local_md5 = ''
        with open(local_file, 'rb') as stream:
            local_md5 = hashlib.md5(stream.read()).digest()
        
        processed_md5 = ''
        with open(processed_file, 'rb') as stream:
            processed_md5 = hashlib.md5(stream.read()).digest()
        
        self.assertEqual(local_md5, processed_md5)
        

if __name__ == "__main__":
    unittest.main()


from dhi.platform.authentication import ApiKeyIdentity, InteractiveIdentity
import os

TEST_ENVIRONMENT = os.environ.get("TESTENVIRONMENTTARGET", "prod")

TEST_CUSTOMER_ID = None # "TODO: provide valid customer id"

TEST_API_KEY = os.environ.get("OPENAPIKEY") # "TODO: provide valid api key"

if TEST_API_KEY is None:
    TEST_IDENTITY = InteractiveIdentity(environment=TEST_ENVIRONMENT)
else:
    TEST_IDENTITY = ApiKeyIdentity(customer_id=TEST_CUSTOMER_ID, apikey=TEST_API_KEY, environment=TEST_ENVIRONMENT)



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.metadatahelper import MetadataClientV2Helper, MetadataClientV2Contracts
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--uploadurl", help="Upload url")
    parser.add_argument("--uploadfile", help="Upload file")
    parser.add_argument("--filename", help="File name")
    parser.add_argument("--destinationpath", help="Destination path")
    parser.add_argument("--createdestinationpathifnotexists", action="store_true", help="Create destination path")

def main():
    args = ClientArgs.ParseForProject(description="Upload file", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    tmpurl = None
    if args.uploadfile:
        tmpurl = MetadataClientV2Helper.UploadFromFile(args.uploadfile, clientv2)
        args.uploadurl = tmpurl

    input = MetadataClientV2Contracts.PrepareUploadStagedFilesInput(
        ClientArgs.LoadJson(args.inputfile),
        args.uploadurl,
        args.filename,
        args.destinationpath,
        args.createdestinationpathifnotexists)
    response = clientv2.UploadStagedFilesV2(input, args.projectid)

    tablefmt1 = "{!s:32}\t{}"
    tablefields1 = ["datasetId", "fileName"]
    tablefmt2 = "{!s:32}\t{}"
    tablefields2 = ["fileName", "message"]
    Format.FormatResponseItems(response, lambda r: [(r.Body.get("datasets"), tablefmt1, tablefields1), (r.Body.get("failures"), tablefmt2, tablefields2)], args.format)

    if tmpurl:
        MetadataClientV2Helper.DeleteBlob(tmpurl)

if __name__ == '__main__':
    main()



import unittest
import datetime
from dhi.platform.generated.enginegen import EngineInputItemV2, EngineModelItemV2
from .testcredentials import TEST_IDENTITY
from dhi.platform import eventing, metadata, transfer, engine
import os


class TestEngineTest(unittest.TestCase):

    _verbosity = 0
    _project_id = None
    _identity = None
    _test_data_dir = None
    
    def setUp(self) -> None:
        if not self._identity:
            self._identity = TEST_IDENTITY
        
        self._engine = engine.EngineClient(
            verbose=self._verbosity, 
            identity=self._identity
        )

        self._metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._transferclient = transfer.TransferClient(verbose=self._verbosity, identity=TEST_IDENTITY)

        self._test_data_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
        self._stamp = stamp = datetime.datetime.utcnow().strftime('%Y%m%d%H%M%S')
        if not self._project_id:
            name = 'Python test engine' + self._stamp
            projectInput = metadata.CreateProjectInput(name, 'Project created by Python SDK test')
            project = self._metadataclient.create_project(projectInput)
            self._project_id = project.id

    def tearDown(self) -> None:
        super().tearDown()
        if self._project_id:
            self._metadataclient.delete_project(self._project_id, permanently=True)
        
    def test_engine_execution_is_ok(self):

        input = [
            EngineInputItemV2("https://coreenginedev0inputs.blob.core.windows.net/data/lake.m21fm", engine="FemEngineHD"),
            EngineInputItemV2("https://coreenginedev0inputs.blob.core.windows.net/data/lake.mesh")
        ]

        pool_type = "VM-S-5" 
        # you can list different pool types, e.g. self._engine.list_configurations(project_id)[0].poolType

        engine_subscription = eventing.EngineSubscription()
        
        event_subscription = eventing.EventSubscriptionBuilder(self._metadataclient, self._project_id)\
            .with_message_handler(lambda m: print(m))\
            .with_on_close_handler(lambda: print("connection closed"))\
            .with_eventing_error_handler(lambda: print("An error during eventing"))\
            .with_engine_handler(engine_subscription, lambda m: print("Success", m), lambda m: print("Failure", m), lambda m: print("Cancelled", m))\
            .build()
        
        parameters = engine.EngineExecutionInputBuilder()\
            .with_input(input)\
            .with_options(pool_type, node_count=1)\
            .with_max_execution_elapsed_time_hours(1)\
            .build()
        
        event_subscription.start_and_subscribe()

        execution = self._engine.run_execution(self._project_id, parameters, engine_subscription)

        engine_subscription.wait()
        event_subscription.stop()

        executionDone = self._engine.get_execution(self._project_id, execution.executionId)
        
        self.assertEqual(execution.executionId, executionDone.executionId)    

    def test_engine_execution_with_platform_data_is_ok(self):
        
        print("Uploading some data...")

        transfer_process1 = self._transferclient.create_url_import(
            self._project_id, "https://coreenginedev0inputs.blob.core.windows.net/data/lake.m21fm", "lake.m21fm")\
            .execute_and_wait()
        
        print(f"Uploaded dataset {transfer_process1.dataset_id}")

        transfer_process2 = self._transferclient.create_url_import(
            self._project_id, "https://coreenginedev0inputs.blob.core.windows.net/data/lake.mesh", "lake.mesh")\
            .execute_and_wait()

        print(f"Uploaded dataset {transfer_process2.dataset_id}")

        models = [
            EngineModelItemV2(modelFileName="lake.m21fm", engine="FemEngineHD", overwriteResultsIfExists=True, resultsRelativePath="results")
        ]

        pool_type = "VM-S-5" 
        # you can list different pool types, e.g. self._engine.list_configurations(project_id)[0].poolType
      
        engine_subscription = eventing.EngineSubscription()

        event_subscription = eventing.EventSubscriptionBuilder(self._metadataclient, self._project_id)\
            .with_message_handler(lambda m: print(m))\
            .with_on_close_handler(lambda: print("connection closed"))\
            .with_eventing_error_handler(lambda: print("An error during eventing"))\
            .with_engine_handler(engine_subscription, lambda m: print("Success", m), lambda m: print("Failure", m), lambda m: print("Cancelled", m))\
            .build()
        
        parameters = engine.EngineExecutionInputParametersBuilder()\
            .with_models(models)\
            .with_options(pool_type, node_count=1)\
            .with_max_execution_elapsed_time_hours(1)\
            .build()
        
        event_subscription.start_and_subscribe()

        execution = self._engine.run_execution_with_platform_data(self._project_id, parameters, engine_subscription)

        engine_subscription.wait()
        event_subscription.stop()

        executionDone = self._engine.get_execution(self._project_id, execution.executionId)
        
        self.assertEqual(execution.executionId, executionDone.executionId)

if __name__ == "__main__":
    unittest.main()


# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from .base.client import Contracts


class TimeSeriesV2Contracts(Contracts):
    @classmethod
    def PrepareCreateTimeseriesDatasetV2Input(cls, input=None, schemaproperties=None, name=None, description=None, metadata=None, properties=None):
        body = input.copy() if input else {}
        tssch = cls.PrepareField(body, "timeSeriesSchema", {})
        cls.SetBodyField(tssch, "properties", schemaproperties, [])
        dsp = cls.PrepareField(body, "datasetProperties", {})
        cls.SetBodyField(dsp, "name", name)
        cls.SetBodyField(dsp, "description", description)
        cls.SetBodyField(dsp, "metadata", metadata, {})
        cls.SetBodyField(dsp, "properties", properties, {})
        return body

    @classmethod
    def PrepareAddTimeSeriesV2Input(cls, input=None, id=None, name=None, unit=None, item=None, datatype=None, timeseriestype=None, properties=None, datafileds=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "id", id)
        i = cls.PrepareField(body, "item", {})
        cls.SetBodyField(i, "name", name)
        cls.SetBodyField(i, "unit", unit)
        cls.SetBodyField(i, "item", item)
        cls.SetBodyField(i, "dataType", datatype)
        cls.SetBodyField(i, "timeSeriesType", timeseriestype)
        cls.SetBodyField(body, "properties", properties, {})
        cls.SetBodyField(body, "dataFileds", datafileds, [])
        return body


if __name__ == '__main__':
    print(__file__)
    print(dir())



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.timeseriesgen import TimeSeriesGenClientV2
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParseForDatasetList(description="Get time series dataset details", defaultformat="yaml")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = TimeSeriesGenClientV2(**vars(args))

    responses = (clientv2.GetTimeSeriesDatasetV2(args.projectid, id) for id in args.datasetids)

    tablefmt = "{!s:32}\t{}"
    tablefields = ["id", "items"]
    Format.FormatResponses(responses, lambda r: r.Body, args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParsePlatform(description="List deleted datasets from all projects")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    response = clientv2.GetRecyclablesFromAllProjectsV2()

    tablefmt = "{!s:32}\t{!s:32}\t{!s:10}\t{}"
    tablefields = ["id", "projectId", "datasetType", "name"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



# Generated using openapi2py.py
# openapi2py.py "--dhiservice" "tiles" "--classname" "TilesGenClientV" "-r" "projectid" "-i" "datasetid=id" "--out" "C:\work\devops\mike-platform-sdk-py\src\dhi\platform\generated\tilesgen.py" "https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/tiles/v1"
# 2022-01-13 19:04:40.570778Z
from typing import Any, Dict, List, NewType, Type, TypeVar, Union
from enum import Enum
import attr
from ..base.client import DataContract, PlatformClient, Response


# https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/tiles/v1
# DHI Tiles API - Version 1
# API for getting raster map tiles
# 1

class TilingSchemeTypeV1(str, Enum):
    WMTS = "WMTS"
    XYZ = "XYZ"
    def __str__(self) -> str:
        return str(self.value)

ProblemDetailsV1Type = TypeVar("ProblemDetailsV1Type", bound="ProblemDetailsV1")

@attr.s(auto_attribs=True)
class ProblemDetailsV1(DataContract):
    type: str = None
    title: str = None
    status: int = None
    detail: str = None
    instance: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ProblemDetailsV1Type, src_dict: Dict[str, Any]) -> ProblemDetailsV1Type:
        obj = ProblemDetailsV1()
        obj.load_dict(src_dict)
        return obj

ValidationProblemDetailsV1Type = TypeVar("ValidationProblemDetailsV1Type", bound="ValidationProblemDetailsV1")

@attr.s(auto_attribs=True)
class ValidationProblemDetailsV1(ProblemDetailsV1):
    errors: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = ProblemDetailsV1.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ValidationProblemDetailsV1Type, src_dict: Dict[str, Any]) -> ValidationProblemDetailsV1Type:
        obj = ValidationProblemDetailsV1()
        obj.load_dict(src_dict)
        return obj

class RenderOutOfRangeTransparentV1(str, Enum):
    NONE = "None"
    ABOVE = "Above"
    BELOW = "Below"
    ABOVEANDBELOW = "AboveAndBelow"
    def __str__(self) -> str:
        return str(self.value)

class TilingAggregationMethodV1(str, Enum):
    MEAN = "Mean"
    MAX = "Max"
    def __str__(self) -> str:
        return str(self.value)

ItemToBandMappingV1Type = TypeVar("ItemToBandMappingV1Type", bound="ItemToBandMappingV1")

@attr.s(auto_attribs=True)
class ItemToBandMappingV1(DataContract):
    itemName: str = None
    minValue: float = None
    maxValue: float = None
    bandIndex: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ItemToBandMappingV1Type, src_dict: Dict[str, Any]) -> ItemToBandMappingV1Type:
        obj = ItemToBandMappingV1()
        obj.load_dict(src_dict)
        return obj

class ETilingOutputTypeV1(str, Enum):
    RAWDATA = "RawData"
    BINNEDDATA256 = "BinnedData256"
    PNG = "PNG"
    WEBP = "WEBP"
    def __str__(self) -> str:
        return str(self.value)

TilingSchemeV1Type = TypeVar("TilingSchemeV1Type", bound="TilingSchemeV1")

@attr.s(auto_attribs=True)
class TilingSchemeV1(DataContract):
    schemeType: TilingSchemeTypeV1 = None
    itemName: str = None
    itemBandMapping: List[ItemToBandMappingV1] = None
    schemeId: str = None
    outputType: ETilingOutputTypeV1 = None
    aggregationMethod: TilingAggregationMethodV1 = None
    tileWidth: int = None
    tileHeight: int = None
    zoomLevels: List[int] = None
    minValue: float = None
    maxValue: float = None
    renderValuesOutOfRangeAsTransparent: RenderOutOfRangeTransparentV1 = None
    colors: List[str] = None
    maxRasterWidth: int = None
    layerIdx: int = None
    compressionLevel: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: TilingSchemeV1Type, src_dict: Dict[str, Any]) -> TilingSchemeV1Type:
        obj = TilingSchemeV1()
        obj.load_dict(src_dict)
        return obj

GeneratedTilingSchemeV1Type = TypeVar("GeneratedTilingSchemeV1Type", bound="GeneratedTilingSchemeV1")

@attr.s(auto_attribs=True)
class GeneratedTilingSchemeV1(TilingSchemeV1):
    resolutions: List[float] = None
    scaleDenominators: List[float] = None
    matrixWidths: List[int] = None
    matrixHeights: List[int] = None
    srid: int = None
    sridAuthName: str = None
    hasLayers: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = TilingSchemeV1.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GeneratedTilingSchemeV1Type, src_dict: Dict[str, Any]) -> GeneratedTilingSchemeV1Type:
        obj = GeneratedTilingSchemeV1()
        obj.load_dict(src_dict)
        return obj

DatasetTilingMetadataV1Type = TypeVar("DatasetTilingMetadataV1Type", bound="DatasetTilingMetadataV1")

@attr.s(auto_attribs=True)
class DatasetTilingMetadataV1(DataContract):
    nativeExtent: str = None
    wgS84Extent: str = None
    tilingSchemes: List[GeneratedTilingSchemeV1] = None
    tileStorageSizeKB: int = None
    timeStamps: List[str] = None
    layerIds: List[int] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: DatasetTilingMetadataV1Type, src_dict: Dict[str, Any]) -> DatasetTilingMetadataV1Type:
        obj = DatasetTilingMetadataV1()
        obj.load_dict(src_dict)
        return obj

class TilesGenClientV1(PlatformClient):
    def __init__(self, inspectFnc=PlatformClient.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, includeheaders=PlatformClient.GetServiceHeaders("tiles"), **kwargs)

    def GetDatasetMetadata(self, projectid, id) -> Response:
        """Tile

        GET /api/tiles/dataset/{id}/metadata
        """
        return self.GetRequest(f"/api/tiles/dataset/{id}/metadata", None, api_version="1", projectid=projectid, datasetid=id)

    def GetEmptyTile(self, projectid, id, schemeid) -> Response:
        """Tile

        GET /api/tiles/dataset/{id}/scheme/{schemeId}/empty-tile
        """
        return self.GetRequest(f"/api/tiles/dataset/{id}/scheme/{schemeid}/empty-tile", None, api_version="1", projectid=projectid, datasetid=id)

    def GetTile(self, projectid, id, schemeid, tilematrix, tilerow, tilecol, layer=None, timestep=None) -> Response:
        """Tile

        GET /api/tiles/dataset/{id}/scheme/{schemeId}/{tileMatrix}/{tileRow}/{tileCol}
        """
        queryparams = self.GetQueryParams(layer=layer, timeStep=timestep)
        return self.GetRequest(f"/api/tiles/dataset/{id}/scheme/{schemeid}/{tilematrix}/{tilerow}/{tilecol}", queryparams, api_version="1", projectid=projectid, datasetid=id)

    def GetWmtsRequest(self, projectid, id, service=None, request=None) -> Response:
        """Tile

        GET /api/tiles/dataset/{id}/wmts
        """
        queryparams = self.GetQueryParams(service=service, request=request)
        return self.GetRequest(f"/api/tiles/dataset/{id}/wmts", queryparams, api_version="1", projectid=projectid, datasetid=id)

    def GetVectorTile(self, projectid, id, zoom, tilecol, tilerow) -> Response:
        """VectorTiles

        GET /api/vectortiles/dataset/{id}/{zoom}/{tileCol}/{tileRow}.vector.pbf
        """
        return self.GetRequest(f"/api/vectortiles/dataset/{id}/{zoom}/{tilecol}/{tilerow}.vector.pbf", None, api_version="1", projectid=projectid, datasetid=id)



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("transferids", metavar="transferid", help="Transfer id", nargs="+")

def main():
    args = ClientArgs.ParsePlatform(description="Transfers info", init=initParser, defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    responses = (clientv2.GetTransferV2(id) for id in args.transferids)

    tablefmt = "{!s:32}\t{!s:32}\t{!s:16}\t{!s:10}\t{!s:20}\t{!s:26}\t{}"
    tablefields = ["id", "projectId", "type", "status", "format", "createdAt", "updatedAt"]
    Format.FormatResponses(responses, lambda r: r.Body, args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



import datetime
from dhi.platform.base.exceptions import MikeCloudException
from .constants import DATETIMEFORMAT_SECONDS_URL


def sanitize_from_to(from_:datetime.datetime=None, to:datetime.datetime=None):
    #if not (from_ or to):
    #    raise MikeCloudException("At least one of from_ and to parameters must be specified")
    
    if (from_ and to) and  (to < from_):
        raise MikeCloudException("Parameter from_ must be lower than parameter to")

    if from_:
        from_ = datetime.datetime.strftime(from_, DATETIMEFORMAT_SECONDS_URL)
    
    if to:
        to = datetime.datetime.strftime(to, DATETIMEFORMAT_SECONDS_URL)
    
    return (from_, to)


# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import argparse, inspect, json, itertools, os, http.client, urllib.parse, sys, yaml

class Format:
    @staticmethod
    def PrintHeader(fmt, fields, file=sys.stdout):
        print(fmt.format(*fields), file=file)

    @staticmethod
    def PrintFields(fmt, obj, fields, file=sys.stdout):
        values = [obj.get(x) for x in fields]
        print(fmt.format(*values), file=file)

    @staticmethod
    def PrintItems(items, fmt, fields, printlen=True, printheader=True, file=sys.stdout):
        if printlen:
            print(f"{len(items)}", file=file)
        if printheader:
            Format.PrintHeader(fmt, fields, file=file)
        for i in items:
            Format.PrintFields(fmt, i, fields, file=file)

    @staticmethod
    def PrintObjFields(obj, fmt, fields, file=sys.stdout):
        for f in fields:
            print(f"{fmt.format(f)}\t{obj.get(f)}", file=file)

    @classmethod
    def DumpJson(cls, obj, file=sys.stdout):
        if obj:
            json.dump(obj, file, indent=True)
    
    @classmethod
    def DumpYaml(cls, obj, file=sys.stdout):
        if obj:
            yaml.dump(obj, file, default_flow_style=False)

    @classmethod
    def DumpPlain(cls, obj, fmt=None, file=sys.stdout, prefix=""):
        lines = cls.__GetLines(obj, prefix) if obj else None
        if lines:
            if not fmt:
                maxLen = max([len(x) for x, _ in lines])
                fmt = f"{{!s:{maxLen}}}"
            for p, v in lines:
                print(f"{fmt.format(p)}\t{v}", file=file)
    @classmethod
    def __GetLines(cls, obj, prefix):
        lines = []
        if isinstance(obj, dict):
            prefixp = (prefix+".") if prefix else prefix
            for i, v in obj.items():
                lines.extend(cls.__GetLines(v, f"{prefixp}{i}"))
        elif isinstance(obj, list):
            for i, v in enumerate(obj):
                lines.extend(cls.__GetLines(v, f"{prefix}[{i}]"))
        else:
            lines = [(prefix, obj)]
        return lines

    @classmethod
    def DumpPlain2(cls, obj, fmt="{}", file=sys.stdout, prefix=""):
        stack = [(prefix, obj)]
        while stack:
            prefix, obj = stack.pop()
            if isinstance(obj, dict):
                for i, v in obj.items():
                    stack.append((prefix+"."+i, v))
            elif isinstance(obj, list):
                for v in obj:
                    stack.append((prefix+"[]", v))
            else:
                print(f"{fmt.format(prefix)}\t{obj}", file=file)

    @staticmethod
    def FormatResponses(responses, gettableoutput=None, format=None, tablefmt=None, tablefields=None, file=sys.stdout, errfile=sys.stderr):
        #if format=="yaml":
        #    Format.DumpYaml([r.Body for r in responses], file=file)
        #elif format=="json":
        #    Format.DumpJson([r.Body for r in responses], file=file)
        #elif format=="plain":
        #    Format.DumpPlain([r.Body for r in responses], file=file)
        #else:
        printclosingbracket = False
        for index, response in enumerate(responses):
            if response.IsOk:
                if format=="json":
                    print("[" if index == 0 else ",", file=file)
                    Format.DumpJson(response.Body, file=file)
                    printclosingbracket = True
                elif format=="yaml":
                    Format.DumpYaml([response.Body], file=file)
                elif format=="plain":
                    Format.DumpPlain(response.Body, prefix=f"[{index}]", file=file)
                elif format:
                    output = gettableoutput(response) if gettableoutput else None
                    if output and tablefields:
                        if index == 0:
                            Format.PrintHeader(tablefmt, tablefields, file=file)
                        if isinstance(output, list):
                            Format.PrintItems(output, tablefmt, tablefields, False, False, file=file)
                        else:
                            Format.PrintFields(tablefmt, output, tablefields, file=file)
                    else:
                        Format.DumpPlain(output, prefix=f"[{index}]", file=file)
            else:
                response.Dump(file=errfile)
        if printclosingbracket:
            print("]", file=file)

    @staticmethod
    def FormatResponse(response, gettableoutput=None, format=None, tablefmt=None, tablefields=None, file=sys.stdout, errfile=sys.stderr):
        if response.IsOk:
            if format=="json":
                Format.DumpJson(response.Body, file=file)
            elif format=="yaml":
                Format.DumpYaml(response.Body, file=file)
            elif format=="plain" or format and not tablefields:
                Format.DumpPlain(response.Body, file=file)
            elif format and tablefields and gettableoutput:
                output = gettableoutput(response)
                if isinstance(output, list):
                    if not tablefmt:
                        tablefmt = "\t".join((f"{{!s:{len(x)}}}" for x in tablefields))
                    Format.PrintItems(output, tablefmt, tablefields, file=file)
                else:
                    if not tablefmt:
                        maxlen = max([len(x) for x in tablefields])
                        tablefmt = f"{{!s:{maxlen}}}"
                    Format.PrintObjFields(output, tablefmt, tablefields, file=file)
        else:
            response.Dump(file=errfile)

    @staticmethod
    def FormatResponseItems(response, gettableoutputs=None, format=None, file=sys.stdout, errfile=sys.stderr):
        if response.IsOk:
            if format=="json":
                Format.DumpJson(response.Body, file=file)
            elif format=="plain":
                Format.DumpPlain(response.Body, file=file)
            elif format and gettableoutputs:
                outputs = gettableoutputs(response)
                for (output, tablefmt, tablefields) in outputs:
                    if not tablefmt:
                        tablefmt = "\t".join((f"{{!s:{len(x)}}}" for x in tablefields))
                    Format.PrintItems(output, tablefmt, tablefields, file=file)
        else:
            response.Dump(file=errfile)

if __name__ == '__main__':
    print(__file__)
    print(dir())



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.metadatahelper import MetadataClientV3Contracts
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--actiontype", choices=["PathActionCreate", "PathActionCreateIfNotExists", "PathActionDelete"], action="append", help="File/folder path")
    parser.add_argument("--path", action="append", help="File/folder path")
    parser.add_argument("--isfolder", type=bool, action="append", help="Project description")
    parser.add_argument("--defaultaccesslevel", choices=["Confidential", "Private", "Shared"], help="Default access level for newly created subprojects/folders")
    parser.add_argument("--sastokenexpiration", help="Project members array JSON, []")

def main():
    args = ClientArgs.ParseForProject(description="Prepare files and subfolders", init=__initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    input = MetadataClientV3Contracts.PreparePrepareHierarchyInput(
        ClientArgs.LoadJson(args.inputfile),
        actiontype=args.actiontype,
        path=args.path,        
        isfolder=args.isfolder,
        defaultaccesslevel=args.defaultaccesslevel,
        sastokenexpiration=args.sastokenexpiration)
    response = clientv3.PrepareHierarchy(input, args.projectid)

    tablefields = ["id", "name"]
    Format.FormatResponse(response, lambda r: r.Body, args.format, None, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("--fromtime", help="From time")
    parser.add_argument("--totime", help="To time")
    parser.add_argument("--status", help="Status filter")
    parser.add_argument("--offset", default=0, help="Output offset")
    parser.add_argument("--limit", default=50, help="Limit output size in one call")

def main():
    args = ClientArgs.ParseForProject(description="List transfers", init=initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    response = clientv2.GetProjectTransferListV2(args.projectid, args.fromtime, args.totime, args.status, args.offset, args.limit)

    tablefmt = "{!s:32}\t{!s:10}\t{!s:26}\t{!s:16}\t{}"
    tablefields = ["id", "status", "createdAt", "type", "format"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
import datetime
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.enginehelper import EnginesClientV2Contracts
from dhi.platform.fmt import Format
from dhi.platform.generated.enginegen import EngineGenClientV2

def __initParser(parser):
    parser.add_argument("--starttime", help="Start time")
    parser.add_argument("--endtime", help="End time")


def main():
    args = ClientArgs.ParseForProject(description="Get list of execution runs", init=__initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = EngineGenClientV2(**vars(args))

    def getResponses():
        cursor = None
        nextlink = None
        while True:
            #if nextlink:
            #    response = clientv2.GetExecutions(args.projectid, nextlink=nextlink)
            if cursor:
                response = clientv2.GetExecutions(args.projectid, cursor=cursor)
            else:
                response = clientv2.GetExecutions(args.projectid, starttime=args.starttime, endtime=args.endtime, cursor=cursor)
            yield response
            cursor = response.Body.get("cursor")
            nextlink = response.Body.get("@nextLink")
            listobjdata = response.Body.get("data")
            if (not cursor and not nextlink) or not listobjdata:
                break
    responses = getResponses()

    def getList(r):
        l = [EnginesClientV2Contracts.SetComputedDuration(EnginesClientV2Contracts.SetComputedPreparation(x)) for x in r.Body.get("data")]
        l.sort(reverse=True, key=lambda x: x.get("createdAt"))
        return l
    tablefmt = "{!s:36}\t{!s:25}\t{!s:28}\t{!s:14}\t{!s:14}\t{!s:9}\t{}"
    tablefields = ["executionId", "status", "createdAt", "preparation", "duration", "nodeCount", "engines"]
    Format.FormatResponses(responses, getList, args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.enginegen import EngineGenClientV2
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("engines", metavar="engine", help="Engine name", nargs="+")

def main():
    args = ClientArgs.ParsePlatform(description="Get engine info", init=__initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = EngineGenClientV2(**vars(args))

    responses = (clientv2.GetEngine(name) for name in args.engines)

    tablefmt = "{!s:10}\t{}"
    tablefields = ["name", "versions"]
    Format.FormatResponses(responses, lambda r: r.Body, args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
import importlib
import os.path, sys
from dhi.platform.args import ClientArgs

def main():
    if len(sys.argv) > 1:
        module = importlib.import_module(sys.argv[1])
        dirname = module.__path__[0]
    else:
        dirname = os.path.dirname(__file__)
    ClientArgs.ListPackageModules(dirname)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.metadatahelper import MetadataClientV3Contracts
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--adduserid", help="User id")
    parser.add_argument("--addrole", help="User role")

def main():
    args = ClientArgs.ParseForProject(description="Add project member", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    input = MetadataClientV3Contracts.PrepareCreateProjectMemberInput(
        ClientArgs.LoadJson(args.inputfile), userid=args.adduserid, role=args.addrole)
    response = clientv3.CreateProjectMemberV3(input, args.projectid)

    Format.FormatResponse(response, None, args.format)

if __name__ == '__main__':
    main()



from azure.storage.blob import BlobClient
from .metadata import MetadataClientV2

class TransferUploadHelper:
    def __init__(self, metadataclient2:MetadataClientV2) -> None:
        self._metadata2 = metadataclient2

    def stage_file(self, file_path:str, max_parallelism=2, block_size=20*1024*1024) -> str:
        """
        Prepare a file for import into the platform by uploading it into a staging area.

        :param file_path: Input file
        :param max_parallelism: maximum concurrency to use when files are over 64 MB
        :param block_size: max_block_size - The maximum chunk size for uploading a block blob in chunks.
        :return: Url with the staged file (the url can be used as an input for transfer operations)
        :rtype: str
        """
        blob_url = ""
        with open(file_path, "rb") as data:
            blob_url = self.stage_stream(data, max_parallelism=max_parallelism, block_size=block_size)
        return blob_url
    
    def stage_stream(self, stream, max_parallelism:int=2, block_size:int=20*1024*1024) -> str:
        """
        Prepare a stream for import into the platform by uploading it into a staging area.

        :param stream: Input stream
        :param max_parallelism: maximum concurrency to use when files are over 64 MB
        :param block_size: max_block_size - The maximum chunk size for uploading a block blob in chunks.
        :return: Url with the staged stream (the url can be used as an input for transfer operations)
        :rtype: str
        """
        # TODO: there is room for improvement for parallizing block uploads as in .net SDK
        blob_url = self._metadata2.GetUploadUrlV2().Body["data"]
        blob = BlobClient.from_blob_url(blob_url, max_block_size=block_size)
        blob.upload_blob(stream, max_concurrency=max_parallelism)
        return blob_url


#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.rawgen import RawGenClientV2
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParseForDatasetPos(description="Get dataset blocks", defaultformat="yaml")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = RawGenClientV2(**vars(args))

    response = clientv2.GetDatasetBlocksChecksums(args.projectid, args.datasetid)

    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format)

if __name__ == '__main__':
    main()



import unittest
import datetime
from unittest.case import _AssertRaisesContext
import uuid
from dhi.platform import metadata, timeseries
from dhi.platform.base.exceptions import MikeCloudRestApiException
from .testcredentials import TEST_IDENTITY

class TestCreateProject(unittest.TestCase):

    _verbosity = 0
    
    def setUp(self) -> None:
        self._metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._timeseriesclient = timeseries.TimeSeriesClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._projects_to_delete = []

    def tearDown(self) -> None:
        super().tearDown()
        for p in self._projects_to_delete:
            try:
                self._metadataclient.delete_project(p, permanently=True)
            except:
                pass
        self._projects_to_delete = []

    def _create_project(self, title_root="Python test ", description="Project created by Python SDK test"):
        stamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
        name = title_root + stamp
        projectInput = metadata.CreateProjectInput(name, description)
        project = self._metadataclient.create_project(projectInput)
        return project

    def _create_ts_dataset(self, project_id, name, description):
        dataset = self._timeseriesclient.create_timeseries_dataset_from_schema(project_id, name, description)
        return dataset.id

    def test_get_and_delete_dataset_is_ok(self):
        project = self._create_project()
        self._projects_to_delete.append(project.id)

        ds_name = "Py TS"
        ds_description = "Test TS dataset"
        dataset_id = self._create_ts_dataset(project.id, ds_name, ds_description)

        dataset = self._metadataclient.get_dataset(dataset_id)
        self.assertIsNotNone(dataset)
        self.assertTrue(ds_name, dataset.name)
        self.assertTrue(ds_name, dataset.description)

        deleted = self._metadataclient.delete_dataset(dataset_id, permanently=True)
        self.assertTrue(deleted)

    def test_list_datasets_is_ok(self):
        project = self._create_project()
        self._projects_to_delete.append(project.id)

        tsd1_id = self._create_ts_dataset(project.id, "TSD 1", "ts dataset 1")
        tsd2_id = self._create_ts_dataset(project.id, "TSD 2", "ts dataset 2")
        
        datasets = [*self._metadataclient.list_datasets(project.id)]

        self.assertTrue(datasets)
        self.assertEqual(len(datasets), 2)

    def test_list_datasets_recursive_is_ok(self):
        project = self._create_project()
        self._projects_to_delete.append(project.id)

        tsd1_id = self._create_ts_dataset(project.id, "TSD 1", "ts dataset 1")
        tsd2_id = self._create_ts_dataset(project.id, "TSD 2", "ts dataset 2")
        
        datasets = [*self._metadataclient.list_datasets_recursive(project.id)]

        self.assertTrue(datasets)
        self.assertEqual(len(datasets), 2)

    def test_update_dataset_is_ok(self):
        project = self._create_project()
        self._projects_to_delete.append(project.id)
        name = "TSD 1"
        description = "ts dataset 1"
        tsd1_id = self._create_ts_dataset(project.id, name, description)

        metadata = { "foo": "bar", "spam": 1 }

        updated_dataset = self._metadataclient.update_dataset(project.id, tsd1_id, name + "Updated", description + "Updated", metadata)

        self.assertEqual(name + "Updated", updated_dataset.name)
        self.assertEqual(description + "Updated", updated_dataset.description)
        self.assertEqual(metadata, updated_dataset.metadata)

    def test_move_dataset_is_ok(self):
        project = self._create_project()
        self._projects_to_delete.append(project.id)
        project2 = self._create_project()
        self._projects_to_delete.append(project2.id)

        dataset_id = self._create_ts_dataset(project.id, "Py TS", "Test TS dataset")

        moved = self._metadataclient.move_dataset(dataset_id, project2.id)
        self.assertTrue(moved)


if __name__ == "__main__":
    unittest.main()


import unittest
import datetime
import os
import filecmp
from dhi.platform import metadata, transfer
from .testcredentials import TEST_IDENTITY

class TestUploadDownload(unittest.TestCase):

    _verbosity = 0
    
    def setUp(self) -> None:
        self._metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._transferclient = transfer.TransferClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        stamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
        name = 'Python upload/download test ' + stamp
        projectInput = metadata.CreateProjectInput(name, 'Project created from Python')
        self._project = self._metadataclient.create_project(projectInput)

    def tearDown(self) -> None:
        self._metadataclient.delete_project(self._project.id, permanently=True)
        if os.path.exists(self._downloaded_file):
            os.remove(self._downloaded_file)

    def test_file_upload_download_is_ok(self):
        test_data_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
        local_file = os.path.join(test_data_dir, 'esbjergSmall.txt')
        project_id = self._project.id

        dataset_id = self._transferclient.upload_file(local_file, project_id, verbose=True)

        self.assertTrue(dataset_id)

        downloaded_file = self._transferclient.download_file(project_id, dataset_id, local_file + '.downloaded', verbose=True)
        
        self.assertTrue(os.path.exists(downloaded_file))
        self.assertTrue(filecmp.cmp(local_file, downloaded_file, shallow=False))

        self._downloaded_file = downloaded_file

    
if __name__ == "__main__":
    unittest.main()


# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import argparse, inspect, json, itertools, os, pkgutil, sys
from typing import Callable
#from .config import ClientConfig

class ClientArgs:
    @staticmethod
    def ListPackageModules(pkgpath):
        print(f"Commands in {os.path.basename(pkgpath)}:")
        for m in pkgutil.iter_modules([pkgpath]):
            if not m.ispkg:
                print(f"    {m.name}")

    @classmethod
    def LoadJson(cls, inputFile, default=None, encoding="utf-8-sig"):
        if inputFile:
            if inputFile == "-":
                return json.load(sys.stdin)
            else:
                with open(inputFile, "r", encoding=encoding) as inputf:
                    return json.load(inputf)
        return default

    @classmethod
    def DumpJson(cls, obj, outFile):
        json.dump(obj, outFile, indent=True)

    @classmethod
    def LoadJsonStr(cls, str, default=None):
        return json.loads(str) if str else default

    @classmethod
    def ParseBasic(cls, prog=None, description=None, init=None):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserBasic(p, init))

    @classmethod
    def ParseEnvironment(cls, prog=None, description=None, init=None):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserEnvironment(p, init))

    @classmethod
    def ParseApiKey(cls, prog=None, description=None, init=None):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserApiKey(p, init))

    @classmethod
    def ParsePlatform(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserPlatform(p, init, defaultformat))

    @classmethod
    def ParseForProject(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserProject(p, init, defaultformat))

    @classmethod
    def ParseForProjectList(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserProjectList(p, init, defaultformat))

    @classmethod
    def ParseForDataset(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserDataset(p, init, defaultformat))

    @classmethod
    def ParseForDatasetList(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserDatasetList(p, init, defaultformat))

    @classmethod
    def ParseForDatasetListOpt(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserDatasetListOpt(p, init, defaultformat))

    @classmethod
    def ParseForDatasetPos(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserDatasetPos(p, init, defaultformat))

    @classmethod
    def ParseForTimeseriesList(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserTimeseriesList(p, init, defaultformat))

    @classmethod
    def ParseForExecutionRunList(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserExecutionRunList(p, init, defaultformat))

    @classmethod
    def ParseForEngineRun(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserEngineRun(p, init, defaultformat))

    @classmethod
    def ParseForEngineRuns(cls, prog=None, description=None, init=None, defaultformat="table"):
        return cls.__Parse(prog, description, lambda p: cls.__InitParserEngineRuns(p, init, defaultformat))

    @staticmethod
    def GetDefault(name, default=None):
        result = os.environ.get(name)
        return result if result else default

    @staticmethod
    def GetDefaultL(name, default: Callable=None):
        result = os.environ.get(name)
        return result if result else default() if default else None

    @classmethod
    def __Parse(cls, prog=None, description=None, init=None):
        parser = cls.__CreateArgumentParser(prog, description, init)
        try:
            return parser.parse_args()
        except:
            sys.exit(1)

    @classmethod
    def __CreateArgumentParser(cls, prog=None, description=None, init=None):
        parser = argparse.ArgumentParser(prog=prog, description=description, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
        if init:
            init(parser)
        return parser

    @classmethod
    def __InitParserBasic(cls, parser, init=None):
        if init:
            init(parser)
        #parser.add_argument("--dump", dest="dump", help="Dump the output", action="store_true")
        parser.add_argument("-v", "--verbose", default=0, help="Make the operation more talkative (increase verbosity), dump the output", action="count")

    @classmethod
    def __InitParserEnvironment(cls, parser, init=None, initParserBasic=True):
        if init:
            init(parser)
        #def getdefaultenvironment():
        #    tmp = ClientConfig.GetDefaultEnvironmentName(configname=cls.GetDefault("DHICONFIG"))
        #    return tmp if tmp else "prod"
        #parser.add_argument("-e", "--environment", default=cls.GetDefaultL("DHIPLATFORMENV", getdefaultenvironment), help="Environment", choices=["local", "dev", "DEV", "dev0", "DEV0", "test", "preprod", "prod"])
        parser.add_argument("-e", "--environment", default=cls.GetDefault("DHIPLATFORMENV"), help="Environment", choices=["local", "dev", "DEV", "dev0", "DEV0", "test", "preprod", "prod"])
        parser.add_argument("--config", help="Configuration name", default=cls.GetDefault("DHICONFIG"))
        if initParserBasic:
            cls.__InitParserBasic(parser)

    @classmethod
    def __InitParserApiKey(cls, parser, init=None, initParserEnvironment=True, initParserBasic=True):
        if init:
            init(parser)
        parser.add_argument("-a", "--apikey", default=cls.GetDefault("DHIAPIKEY"), help="API key")
        if initParserEnvironment:
            cls.__InitParserEnvironment(parser, None, initParserBasic)
        
    @classmethod
    def __InitParserPlatform(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        # %(default)s
        cls.__InitParserApiKey(parser,None, False, False)
        parser.add_argument("--preferapikey", default=None, help="Prefer API key", action="store_true")
        cls.__InitParserEnvironment(parser, None, False)
        parser.add_argument("-m", "--metaurl", default=cls.GetDefault("DHICOREMETADATAURL"), help="Metadata service URL")
        parser.add_argument("--customerid", default=cls.GetDefault("DHICUSTOMERGUID"), help="Customer id")
        parser.add_argument("--userid", default=cls.GetDefault("DHIUSERID"), help="User id")
        parser.add_argument("--userisadmin", choices=["true", "false"], default=cls.GetDefault("DHIUSERISADMIN"), help="true if user is admin")
        parser.add_argument("--format", help="Output format", choices=["plain", "table", "json", "yaml"], default=cls.GetDefault("DHIOUTPUTFORMAT", defaultformat))
        parser.add_argument("--output", help="Output file", default=cls.GetDefault("DHIOUTPUTFILE"))
        cls.__InitParserBasic(parser)

    @classmethod
    def __InitParserProject(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("-p", "--projectid", default=cls.GetDefault("DHIPROJECTID"), help="Project id")
        cls.__InitParserPlatform(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserProjectList(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("projectids", metavar="projectid", help="Project id", nargs="+")
        cls.__InitParserPlatform(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserDataset(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("-d", "--datasetid", default=cls.GetDefault("DHIDATASETID"), help="Dataset id")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserDatasetList(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("datasetids", metavar="datasetid", help="Dataset id", nargs="+")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserDatasetListOpt(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("datasetids", metavar="datasetid", help="Dataset id", nargs="*")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserDatasetPos(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("datasetid", help="Dataset id")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserTimeseriesList(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("tsids", metavar="tsid", help="Timeseries id", nargs="+")
        cls.__InitParserDataset(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserExecutionRunList(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("executionids", metavar="executionid", help="Execution run id", nargs="+")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserEngineRun(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("-x", "--executionid", default=cls.GetDefault("DHIEXECUTIONID"), help="Execution run id")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

    @classmethod
    def __InitParserEngineRuns(cls, parser, init=None, defaultformat="table"):
        if init:
            init(parser)
        parser.add_argument("-r", "--runno", default=cls.GetDefault("DHIRUNNO"), help="Run number")
        parser.add_argument("-u", "--run", help="Run path", default=".")
        cls.__InitParserProject(parser, defaultformat=defaultformat)

if __name__ == '__main__':
    print(__file__)
    print(dir())



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.rawhelper import RawClientV2Contracts
from dhi.platform.generated.rawgen import RawGenClientV2
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--id", action="append", help="Dataset id")
    parser.add_argument("--sastoken", action="append", help="SAS token")
    parser.add_argument("--name", action="append", help="File name")
    parser.add_argument("--url", action="append", help="Source data URL")
    parser.add_argument("--lastmodified", action="append", help="File last modification time")
    parser.add_argument("--size", action="append", help="File size")
    parser.add_argument("--forcecopy", action="append", help="Set to ")

def main():
    args = ClientArgs.ParseForProject(description="Upload files", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = RawGenClientV2(**vars(args))

    input = RawClientV2Contracts.PrepareUploadBulkFilesInput(
        ClientArgs.LoadJson(args.inputfile),
        id=args.id,
        sastoken=args.sastoken,
        name=args.name,
        url=args.url,
        lastmodified=args.lastmodified,
        size=args.size,
        forcecopy=args.forcecopy)
    response = clientv2.UploadBulkFiles(args.projectid, input)

    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format)

if __name__ == '__main__':
    main()



import datetime
import json
import logging
import threading
from typing import List
from signalrcore import hub_connection_builder
import uuid
from dhi.platform.base.exceptions import MikeCloudException
from dhi.platform.metadata import MetadataClient


class EngineSubscription():
    def __init__(self) -> None:
        self._resourceId = None
        self._resourceIdEvent = threading.Event()
        self._synchronization_event = threading.Event()

    def wait(self):
        self._synchronization_event.wait()

    def set(self):
        self._synchronization_event.set()

    def set_resource_id(self, resourceId):
        self._resourceId = resourceId
        self._resourceIdEvent.set()

    def handle_message(self, event:List[str], on_success_handler, on_failed_handler, on_cancelled_handler):
        """
        This function shuold be used as engine execution message handler to control engine execution process.
        :param synchronization_event: threading.Event to signal finished engine execution
        :param on_success_handler: callable to run on sucess with one parameter (the event)
        :param on_failed_handler: callable to run on failure with one parameter (the event)
        :param on_cancelled_handler: callable to run on cancellation with one parameter (the event)
        """
        if not callable(on_success_handler):
            raise MikeCloudException("Parameters on_success_handler must be callable")

        if not callable(on_failed_handler):
            raise MikeCloudException("Parameter on_failed_handler must be callable")
        
        if not callable(on_cancelled_handler):
            raise MikeCloudException("Parameter on_cancelled_handler must be callable")

        if len(event) !=1:
            raise MikeCloudException("Expected a single event, obtained " + str(len(event)), event)
        event = event[0]

        try:
            event = json.loads(event)
        except Exception as ex:
            raise MikeCloudException("Cannot deserialize event message", ex)

        self._resourceIdEvent.wait()
        if event["type"] == "dhi.waterdata.engines.eventing.contracts.messages.v3.engineexecutionstatuschangedmessage":
            resourceId = event.get("resourceid", None)
            if resourceId == self._resourceId:
                data = event["data"]
                status = data["Status"]
                if data.get("Status", None) == "Success":
                    if data["RunningSetupIndex"] + 1 == data["TotalNumberOfSetups"]:
                        # the execution has finished we can stop the subscription (set subscription connectionid stopped on a synchronization object)
                        on_success_handler(event)
                        self._synchronization_event.set()
                    else:
                        # one stage in execution has finished
                        pass
                if status == "Failed" or status == "Failure":
                    # we can stop the subscription (set subscription connectionid stopped on a synchronization object)
                    on_failed_handler(event)
                    self._synchronization_event.set()
                if status == "Cancelled":
                    # we can stop the subscription (set subscription connectionid stopped on a synchronization object)
                    on_cancelled_handler(event)
                    self._synchronization_event.set()


class EventSubscription():
    def __init__(self, hub_connection) -> None:
        self._hub_connection = hub_connection
        self._hub_connection.on_open(self._subscribe)
        self._subscription_id = str(uuid.uuid4())

    def _subscribe(self):
        sub = {
            "SubscriptionId": self._subscription_id
        }

        if self._filter:
            sub["Filter"] = self._filter

        sub["Condition"] = { 
            "Offset": self._offset
        }

        if self._offset_point:
            sub["Condition"]["OffsetPoint"] = self._offset_point

        self._hub_connection.send("Subscribe", [sub])

    def start_and_subscribe(self, filter:dict=None, offset:int=200, offset_point:str=None) -> str:
        """
        Start the subscription and subscribe to the events.
        :param filter: limit the events to read, e.g. {"Sources": ["/someservice/someoperation/v1/"]}
        :param offset: How many events to skip at the beginning of reading, e.g. 100 = Beginning, 200 = End, 300 = Point
        :param offset_point: Indicates where reading of events should start if offset is 300, e.g. "1057"
        :returns: Subscription ID
        :rtype: str
        """
        self._filter = filter
        self._offset = offset
        self._offset_point = offset_point
        self._hub_connection.start()

    def stop(self):
        """
        Stop the connection
        """
        self._hub_connection.stop()


class EventSubscriptionBuilder():
    def __init__(self, metadata_client:MetadataClient, project_id) -> None:
        self._project_id = project_id
        self._automatic_reconnect_options = {
            "type": "raw",
            "keep_alive_interval": 10,
            "reconnect_interval": 5,
            "max_attempts": 5
        }
        self._logging_level = logging.ERROR
        self._on_close_handler = lambda: None
        self._on_update_handler = lambda m: None
        self._eventing_on_error = lambda m: None
        self._metadata3 = metadata_client

    def _validate(self):
        messages = []

        if not self._automatic_reconnect_options:
            messages.append("Event subscription token is required")

        if messages:
            raise MikeCloudException("Invalid connection", messages)

    def _handle_error_and_stop_events(self, handler):
        if self._engine_message_handler:
            self._engine_message_handler.set()
        handler()
        
    def with_logging_level(self, logging_level=logging.ERROR):
        """Override default logging level"""
        self._logging_level = logging_level
        return self

    def with_on_close_handler(self, handler):
        """Configure a function to run when the connection is closed
        :param handler: callable with zero parameters
        """
        if not callable(handler):
            raise MikeCloudException("Parameter handler must be callable")
        self._on_close_handler = handler
        return self
    
    def with_message_handler(self, handler):
        """Configure a function to run when the connection receives an event
        :param handler: callable with one parameter (the event)
        """
        if not callable(handler):
            raise MikeCloudException("Parameter handler must be callable")
        self._on_update_handler = handler
        return self

    def with_eventing_error_handler(self, handler):
        """Configure a function to run when the event communication fails
        :param handler: callable with zero parameters
        """
        self._eventing_on_error = lambda: self._handle_error_and_stop_events(handler)
        return self

    def with_engine_handler(self, engine_message_handler:EngineSubscription, on_success_handler, on_failed_handler, on_cancelled_handler):
        """Configure engine execution message handling.
        This method is intended to be used only when subscribing for events in the context of DHI MIKE Cloud Engine Execution
        :param synchronization_event: threadding.Event to be used to by the calling code to detect when engine execution has finished
        :param on_success_handler: callable to run on sucess with one parameter (the event)
        :param on_failed_handler: callable to run on failure with one parameter (the event)
        :param on_cancelled_handler: callable to run on cancellation with one parameter (the event)
        """
        if not callable(on_success_handler):
            raise MikeCloudException("Parameters on_success_handler must be callable")

        if not callable(on_failed_handler):
            raise MikeCloudException("Parameter on_failed_handler must be callable")
    
        if not callable(on_cancelled_handler):
            raise MikeCloudException("Parameter on_cancelled_handler must be callable")
        
        self._engine_message_handler = engine_message_handler
        self._engine_on_success_handler = on_success_handler
        self._engine_on_failed_handler = on_failed_handler
        self._engine_on_cancelled_handler = on_cancelled_handler

        return self

    def with_automatic_reconnect_options(self, keep_alive_interval:int=10, reconnect_interval:int=5, max_attempts:int=5, type="raw"):
        """Override default automatic reconnect options"""
        self._automatic_reconnect_options["keep_alive_interval"] = keep_alive_interval
        self._automatic_reconnect_options["reconnect_interval"] = reconnect_interval
        self._automatic_reconnect_options["max_attempts"] = max_attempts
        self._automatic_reconnect_options["type"] = type
        return self
    
    def build(self) -> EventSubscription:
        self._validate()

        url = self._metadata3.get_service_url("pubsub")
        url = url.rstrip("api").rstrip("api/")

        expiration = datetime.timedelta(hours=23)
        token = self._metadata3.get_sas_token_string(self._project_id, None, expiration)

        hub_connection = hub_connection_builder.HubConnectionBuilder()\
            .with_url(url + "/v1/subscribe",
                options={
                    "headers": {
                        "dhi-sas-token": token
                    },
                    "verify_ssl": False # TODO: REMOVE
                })\
            .configure_logging(self._logging_level, socket_trace=True)\
            .with_automatic_reconnect(self._automatic_reconnect_options)\
            .build()

        if self._on_close_handler:
            hub_connection.on_close(self._on_close_handler)

        if self._on_update_handler:
            hub_connection.on("Update", self._on_update_handler)
        
        if self._eventing_on_error:
            hub_connection.on_error(self._eventing_on_error)
        
        if self._engine_message_handler:
            hub_connection.on("Update", lambda m: self._engine_message_handler.handle_message(m, self._engine_on_success_handler, self._engine_on_failed_handler, self._engine_on_cancelled_handler))

        return EventSubscription(hub_connection)



# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from datetime import datetime
import struct
import array
from typing import List, Tuple

from dhi.platform.base.exceptions import MikeCloudException
from dhi.platform.commonmodels import ItemId, UnitId, SpatialFilter, TemporalFilter, VerticalFilter
from dhi.platform.protobufparser import decoder
from dhi.platform.protobufparser.decoderdatetime import DateListDecoder
from dhi.platform.protobufparser.enums import DataBlockIndex, SectionTag, SpatialTags
from .generated.multidimensionalgen import BinaryOptionsV2, BinaryQueryInputV2, ItemFilterV2, MultidimensionalGenClientV2, MultidimensionalGenClientV3, QueryInputV2, SpatialFilterV2, TemporalDomainOutputGetDatasetOutputV3, VerticalFilterV2

class BinaryOptions(BinaryOptionsV2):

    def __init_subclass__(cls) -> None:
        return super().__init_subclass__()

    def get_spatial_rounding_ratio(self):
        if self.coordinatePrecision is None:
            raise MikeCloudException("Cannot get spatial rounding ratio when coordinate precision is None")
        return int(10**self.coordinatePrecision)

    @property
    def encodeCoordinates(self):
        return self.coordinatePrecision is not None
    

class BinaryDataBlock:
    
    def __init__(self, indexes:dict, data) -> None:
        self._indexes = indexes
        self._data = data

    @property
    def indexes(self):
        return self._indexes
    
    def get_data_raw(self):
        return self._data

    def get_data_byte(self):
        return struct.unpack(str(len(self._data)) + 'b', self._data)
    
    def get_data_short_int(self):
        return array.array('h', self._data)

    def get_data_int(self):
        return array.array('l', self._data)

    def get_data_float(self):
        return array.array('f', self._data)

    def get_data_double(self):
        return array.array('d', self._data)
    
    def __repr__(self) -> str:
        return str(self._indexes) + str(self._data[:min(5, len(self._data))]) + ('...' if len(self._data) > 6 else '')


class BinaryMeshPage:
    def __init__(self, nodes, elements, fixed_node_count) -> None:
        self._nodes = nodes
        self._elements = elements
        self._fixed_node_count = fixed_node_count
    
    @property
    def nodes(self) -> Tuple[tuple]:
        """Tuple of nodes in the mesh page"""
        return self._nodes
    
    @property
    def elements(self) -> Tuple[int]:
        """Tuple of elements in the mesh page"""
        return self._elements


class BinaryQueryOutput:

    def __init__(self) -> None:
        self._data_blocks = []
        self._mesh_reader = MeshBinaryReader()
        self._elements = []
        self._mesh_pages = []
        self._time_steps = []
        self._binary_protocol_version = None
        self._srid = None
    
    @property
    def mesh_pages(self) -> Tuple[BinaryMeshPage]:
        return self._mesh_pages

    @property
    def elements(self) -> Tuple[int]:
        return self._elements

    @property
    def srid(self) -> int:
        """Spatial reference id"""
        return self._srid

    @property
    def data_blocks(self) -> Tuple[BinaryDataBlock]:
        return self._data_blocks

    @property
    def binary_protocol_version(self) -> Tuple:
        return self._binary_protocol_version

    @property
    def time_steps(self) -> Tuple[datetime]:
        return self._time_steps

    def add_mesh_page(self, mesh_page):
        self._mesh_pages.append(mesh_page)

    def _parse_header(self, header) -> Tuple:
        return tuple([i for i in header])

    def _read_spatial_block(self, body, pos) -> int:
        tag, pos = decoder.ReadByte(body, pos)

        if tag == SpatialTags.GRID_INDEXES.value:
            output_elements, pos = decoder.ReadArrayInt(body, pos)
            self._elements = output_elements
        elif tag == SpatialTags.MESH_PAGES.value:
            count, pos = decoder._DecodeVarint32(body, pos)
            for i in range(count):
                mesh_page, pos = self._mesh_reader.ReadMeshPage(body, pos, self._binary_options)
                self.add_mesh_page(mesh_page)
        else:
            raise MikeCloudException(f"Invalid spatial tag {tag} while parsing spatial block.")
        
        return pos
    
    def _read_timesteps(self, body, pos) -> int:
        datetimedecoder = DateListDecoder()
        times, pos = datetimedecoder.read_dates(body, pos)
        self._time_steps = times
        return pos
    
    def _read_block_index(self, body, indexes:dict, pos) -> int:
        tag, pos = decoder.ReadByte(body, pos)
        val, pos = decoder._DecodeSignedVarint32(body, pos)
        indexes[DataBlockIndex.from_value(tag)] = val
        return pos

    def _read_data_block(self, body, pos) -> int:
        indexes = {}
        pos = self._read_block_index(body, indexes, pos)
        pos = self._read_block_index(body, indexes, pos)
        pos = self._read_block_index(body, indexes, pos)

        length, pos = decoder._DecodeVarint32(body, pos)
        data = body[pos:pos+length]
        pos = pos+length

        data_block = BinaryDataBlock(indexes, data)

        self._data_blocks.append(data_block)
        return pos
    
    def from_body(self, body, binary_options:BinaryOptions):
        self._binary_options = binary_options
        self._binary_protocol_version = self._parse_header(body[:4])
        
        srid, position = decoder._DecodeVarint32(body, 4)
        self._srid = srid
        
        tag, position = decoder.ReadByte(body, position)
        
        while (position < len(body)) and (tag != SectionTag.CLOSETAG):
            if tag == SectionTag.SPATIAL.value:
                position = self._read_spatial_block(body, position)
            elif tag == SectionTag.TEMPORAL.value:
                position = self._read_timesteps(body, position)
            elif tag == SectionTag.DATABLOCK.value:
                position = self._read_data_block(body, position)
            else:
                raise Exception(f"Unexpected section tag {tag} at {position}")
            
            tag, position = decoder.ReadByte(body, position)
        
        return self

class BinaryMeshElement:
    def __init__(self, id:int, nodes:Tuple[Tuple], ids:Tuple[int]) -> None:
        self._id = id
        self._nodes = nodes
        self._ids = ids

    @property
    def id(self):
        return self._id

    @property
    def nodes(self):
        return self._nodes

    @property
    def ids(self):
        return self._ids


class MeshBinaryReader:
    
    def __init__(self) -> None:
        pass

    def _read_mesh_nodes_encoded(self, body, pos, node_count, spatial_rounding_ratio):
        nodes = []
        x_encoded = decoder.DoubleDecoderWithPrecision(body, pos, spatial_rounding_ratio)
        pos = x_encoded.get_position()
        y_encoded = decoder.DoubleDecoderWithPrecision(body, pos, spatial_rounding_ratio)
        pos = y_encoded.get_position()
        nodes.append((x_encoded.start_value, y_encoded.start_value, 0.0))
        for i in range(node_count - 1):
            x = x_encoded.get_next_value(body, pos)
            pos = x_encoded.get_position()
            y = y_encoded.get_next_value(body, pos)
            pos = y_encoded.get_position()
            node = (x, y, 0.0)
            nodes.append(node)
        return nodes, pos

    def _read_mesh_nodes_as_doubles(self, body, pos, node_count):
        nodes = []
        for i in range(node_count):
            x, pos = decoder.DecodeDouble(body, pos)
            y, pos = decoder.DecodeDouble(body, pos)
            nodes.append((x, y, 0.0))
        return nodes, pos

    def _read_elements(self, nodes, include_mesh_element_id, fixed_node_count, body, pos):
        count, pos = decoder._DecodeVarint32(body, pos)

        elements = []
        id = 0
        for i in range(count):
            
            if include_mesh_element_id:
                delta, pos = decoder.DecodeSInt32(body, pos)
                id = delta if id == 0 else id + delta
            
            if fixed_node_count == 0:
                node_count, pos = decoder.ReadByte(body, pos)
            else:
                node_count = fixed_node_count
            
            ids = []
            for j in range(node_count):
                idj, pos = decoder.ReadByte(body, pos)
                ids.append(idj)
            
            el = BinaryMeshElement(id, nodes, ids)
            elements.append(el)

        return elements, pos

    def ReadMeshPage(self, body, pos, options:BinaryOptions):
        fixed_node_count, pos = decoder.ReadByte(body, pos)
        node_count, pos = decoder._DecodeSignedVarint32(body, pos)
        if options.encodeCoordinates:
            nodes, pos = self._read_mesh_nodes_encoded(body, pos, node_count, options.get_spatial_rounding_ratio())
        else:
            nodes, pos = self._read_mesh_nodes_as_doubles(body, pos, node_count)
        elements, pos = self._read_elements(nodes, options.includeMeshElementId, fixed_node_count, body, pos)
        return BinaryMeshPage(nodes, elements, fixed_node_count), pos


class TemporalDomainOutputGetDatasetOutput(TemporalDomainOutputGetDatasetOutputV3):
    pass


class BinaryQueryInput(BinaryQueryInputV2):
    pass


class MultidimensionalClientV2(MultidimensionalGenClientV2):
    def __init__(self, inspectFnc=MultidimensionalGenClientV2.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, **kwargs)


class MultidimensionalClientV3(MultidimensionalGenClientV3):
    def __init__(self, inspectFnc=MultidimensionalGenClientV3.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, **kwargs)


class MultidimensionalClient():
    
    def __init__(self, inspectFnc=MultidimensionalClientV3.DefaultInspectFnc, **kwargs):
        self._md2 = kwargs.get("MultidimensionalClientV2", MultidimensionalClientV2(inspectFnc, **kwargs))
        self._md3 = kwargs.get("MultidimensionalClientV3", MultidimensionalClientV3(inspectFnc, **kwargs))

    def get_dataset(self, project_id, dataset_id) -> TemporalDomainOutputGetDatasetOutput:
        """
        Get multidimensional dataset details
        
        :param project_id: ID of the project where the dataset is located
        :param dataset_id: ID of the dataset
        :return: multidimensional datset details
        :rtype: TemporalDomainOutputGetDatasetOutput
        """
        response = self._md3.GetDatasetV3(project_id, dataset_id)
        return TemporalDomainOutputGetDatasetOutput.from_dict(response.Body)

    def update_item(self, project_id, dataset_id, item_index:int, name:str, unit:UnitId, item:ItemId) -> TemporalDomainOutputGetDatasetOutput:
        """
        Update a spefific item in a multidimensional dataset
        
        :param project_id: ID of the project where the dataset is located
        :param dataset_id: ID of the dataset
        :param item_index: Index of the item to update
        :param name: Name of the item, you must include the original name if you don't want to update the item
        :param unit: ID of the unit, you must include the original unit if you don't want to update the unit
        :param item: ID of the item, you must include the original item if you don't want to update the item
        :return: multidimensional datset details
        :rtype: TemporalDomainOutputGetDatasetOutput
        """
        input = {
            "name": name,
            "unit": unit,
            "item": item
        }
        response = self._md3.UpdateItemDefinition(project_id, input, dataset_id, item_index)
        return TemporalDomainOutputGetDatasetOutput.from_dict(response.Body)

    def delete_timestep(self, project_id, dataset_id, timestep_index:int) -> bool:
        """
        Delete a timestep from a multidimensional dataset
        
        :param project_id: ID of the project where the dataset is located
        :param dataset_id: ID of the dataset
        :param timestep_index: Index of the timestep to delete
        :return: True if removal was successful, otherwise False
        :rtype: bool
        """
        response = self._md3.DeleteTimestep(project_id, dataset_id, timestepindex=timestep_index)
        return response.IsOk

    def query_timesteps(
        self,
        project_id, dataset_id, 
        spatial_filter:SpatialFilter,
        temporal_filter:TemporalFilter,
        vertical_filter:VerticalFilter=None,
        item_indices:Tuple[int]=(),
        options = BinaryOptions(
            majorVersion=1,
            minorVersion=1,
            coordinatePrecision=5,
            includeMeshElementId=True
        ), 
        include_geometries=True,
        include_values=True,
        output_srid=None
    ) -> BinaryQueryOutput:
        """
        Query multidimensional dataset for a set of time steps
        - POST ​/api​/md​/dataset​/{id}​/binary-query-timesteps

        :param project_id: ID of the project where the dataset resides in 
        :param dataset_id: ID of the dataset to query
        :param spatial_filter: Filter to limit the spatial extent of the query
        :param temporal_filter: Filter to limit the temporal extent of the query
        :param vertical_filter: Filter to limit vertical extent of the query, optional, default is None for full vertical extent
        :param item_indices: Indices of ites to include in the response, optional, default is empty tuple for all items
        :param options: Binary query options
        :param include_geometries: Should the response include element geometries? Optional, default is True
        :param include_values: Should the response include element values? Optional, default is True
        :param output_srid: For meshes, it is possible to specify output srid as the desired coordinate system id of the resulting geometries.
        """
        item_filter = None if not item_indices else ItemFilterV2(item_indices)

        query = QueryInputV2(
            itemFilter = item_filter,
            spatialFilter = spatial_filter,
            temporalFilter = temporal_filter,
            verticalFilter = vertical_filter,
            includeGeometries = include_geometries,
            includeValues = include_values,
            outputSRID = output_srid
        )

        input = BinaryQueryInput(
            query = query,
            options = options
        ).to_dict()

        response = self._md2.GetQueryTimestepResultAsBinaryStream(project_id, input, dataset_id)
        return BinaryQueryOutput().from_body(response.Body, options)
        

    def query_timeseries(self, 
        project_id, dataset_id, 
        spatial_filter:SpatialFilter,
        temporal_filter:TemporalFilter,
        vertical_filter:VerticalFilter=None,
        item_indices:Tuple[int]=(),
        options = BinaryOptions(
            majorVersion=1,
            minorVersion=1,
            coordinatePrecision=5,
            includeMeshElementId=True
        ), 
        include_geometries=True,
        include_values=True,
        output_srid=None
    ) -> BinaryQueryOutput:
        """
        Query multidimensional dataset for a set of time series
        - POST ​/api​/md​/dataset​/{id}​/binary-query-timeseries

        :param project_id: ID of the project where the dataset resides in 
        :param dataset_id: ID of the dataset to query
        :param spatial_filter: Filter to limit the spatial extent of the query
        :param temporal_filter: Filter to limit the temporal extent of the query
        :param vertical_filter: Filter to limit vertical extent of the query, optional, default is None for full vertical extent
        :param item_indices: Indices of ites to include in the response, optional, default is empty tuple for all items
        :param options: Binary query options
        :param include_geometries: Should the response include element geometries? Optional, default is True
        :param include_values: Should the response include element values? Optional, default is True
        :param output_srid: For meshes, it is possible to specify output srid as the desired coordinate system id of the resulting geometries.
        """
        item_filter = None if not item_indices else ItemFilterV2(item_indices)

        query = QueryInputV2(
            itemFilter = item_filter,
            spatialFilter = spatial_filter,
            temporalFilter = temporal_filter,
            verticalFilter = vertical_filter,
            includeGeometries = include_geometries,
            includeValues = include_values,
            outputSRID = output_srid
        )

        input = BinaryQueryInput(
            query = query,
            options = options
        ).to_dict()

        response = self._md2.GetQueryTimeseriesResultAsBinaryStream(project_id, input, dataset_id)
        return BinaryQueryOutput().from_body(response.Body, options)


if __name__ == '__main__':
    print(__file__)
    print(dir())



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParseForProjectList(description="Restore deleted project")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    responses = (clientv2.RestoreProjectV2(id) for id in args.projectids)

    Format.FormatResponses(responses)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.enginegen import EngineGenClientV2
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParsePlatform(description="Get available engines")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = EngineGenClientV2(**vars(args))

    response = clientv2.GetAllEngines()

    tablefmt = "{!s:10}\t{}"
    tablefields = ["name", "versions"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("--nameprefix", help="Name prefix")
    parser.add_argument("--role", help="Role")
    parser.add_argument("--capability", help="Capability")
    parser.add_argument("--sortby", help="Sort by")
    parser.add_argument("--sortorder", help="Sort order")
    parser.add_argument("--limit", default=50, help="Limit output size in one call")

def main():
    args = ClientArgs.ParseForProject(description="List subprojects", init=initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    def getResponses():
        cursor = None
        while True:
            response = clientv3.GetSubProjectListV3(args.projectid, nameprefix=args.nameprefix, role=args.role, capability=args.capability, sortby=args.sortby, sortorder=args.sortorder, cursor=cursor, limit=args.limit)
            yield response
            cursor = response.Body.get("cursor")
            listobjdata = response.Body.get("data")
            if not cursor or not listobjdata:
                break
    responses = getResponses()

    tablefmt = "{!s:32}\t{}"
    tablefields = ["id", "name"]
    Format.FormatResponses(responses, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()
